import torch
import torchvision
from torch.utils.data import DataLoader, random_split
import torchvision.transforms as transforms


class Threshold_noise:
    """Remove intensities under given threshold"""

    def __init__(self, threshold=100.):
        self.threshold = threshold

    def __call__(self, x):
        return torch.where((x <= self.threshold), torch.tensor(0.))

class Log_normalisation:
    """Log normalisation of intensities"""

    def __init__(self, eps=1e-5):
        self.epsilon = eps

    def __call__(self, x):
        return torch.log(x+1+self.epsilon)/torch.log(torch.max(x)+1+self.epsilon)

class Random_shift_rt():
    pass


def load_data(base_dir, batch_size, shuffle=True, noise_threshold=0):
    train_transform = transforms.Compose(
        [transforms.Grayscale(num_output_channels=1),
         transforms.ToTensor(),
         transforms.Resize((224,224)),
         Threshold_noise(noise_threshold),
         Log_normalisation(),
         transforms.Normalize(0.5, 0.5)])
    print('Default train transform')

    val_transform = transforms.Compose(
        [transforms.Grayscale(num_output_channels=1),
         transforms.ToTensor(),
         transforms.Resize((224,224)),
         Threshold_noise(noise_threshold),
         Log_normalisation(),
         transforms.Normalize(0.5, 0.5)])
    print('Default val transform')
    train_dataset = torchvision.datasets.ImageFolder(root=base_dir, transform=train_transform)
    val_dataset = torchvision.datasets.ImageFolder(root=base_dir, transform=val_transform)
    generator1 = torch.Generator().manual_seed(42)
    indices = torch.randperm(len(train_dataset),generator=generator1)
    val_size = len(train_dataset) // 5
    train_dataset = torch.utils.data.Subset(train_dataset, indices[:-val_size])
    val_dataset = torch.utils.data.Subset(val_dataset, indices[-val_size:])

    data_loader_train = DataLoader(
        dataset=train_dataset,
        batch_size=batch_size,
        shuffle=shuffle,
        num_workers=0,
        collate_fn=None,
        pin_memory=False,
    )

    data_loader_test = DataLoader(
        dataset=val_dataset,
        batch_size=batch_size,
        shuffle=shuffle,
        num_workers=0,
        collate_fn=None,
        pin_memory=False,
    )

    return data_loader_train, data_loader_test